#! /usr/bin/perl -w

#
# This script updates grub configuration and initrd images after the
# (un)installation of MicroSoft integration components. It is invoked
# in the post-(un)install scriptlet of the mshvic RPM.
#

use strict;
use diagnostics;
use POSIX;

sub Usage();
sub Clean(@);

my $debug = 0;

Usage() unless (scalar(@ARGV) == 1 or scalar(@ARGV) == 2);

print "\@ARGV @ARGV\n" if $debug;

my $action = $ARGV[0];
Usage unless grep {$_ eq $action} qw(install uninstall clean);

my $kernelvariant = undef;
if (defined $ARGV[1]){
    $kernelvariant = $ARGV[1];
    Usage unless grep {$_ eq $kernelvariant} qw(kernel kernel-xen kernel-PAE);
}

# Check the directory that stores the state files

my $statedir = "/var/run/mshvic";
if (not -d $statedir){
    print STDERR "No directory $statedir found, exiting\n";
    exit 1;
}

# Get installed kernels

my @kernels_to_process = my @all_kernels = ();
opendir(BOOT,"/boot") || die "can't opendir /boot: $!";
for my $kernel (sort map {s/^vmlinuz-//;$_} grep {/^vmlinuz-(\S+)/} readdir(BOOT)){
    push(@all_kernels,$kernel);
    if (defined $kernelvariant){
        if ($kernel =~ /xen$/){
            next unless $kernelvariant eq "kernel-xen";
        }elsif ($kernel =~ /PAE$/){
            next unless $kernelvariant eq "kernel-PAE";
        }else{
            next unless $kernelvariant eq "kernel";
        }
    }
    push(@kernels_to_process,$kernel);
}
closedir BOOT;
print "\@all_kernels        = @all_kernels\n" if $debug;
print "\@kernels_to_process = @kernels_to_process\n" if $debug;

# Housekeeping

Clean(@all_kernels);
exit 0 if $action eq "clean";

# Handle them

my $err = 0;
my $arch = (POSIX::uname())[4];
for my $kernel (@kernels_to_process){
    print "Processing kernel $kernel\n" if $debug;
    
    # Check state file
    if ($action eq "install"){
        next if -f "$statedir/$kernel";
    }else{
        next unless -f "$statedir/$kernel";
    }
    
    # Update grub configuration
    my $timekeeping = ($arch eq "x86_64" ? "divider=10" : "divider=10 clocksource=acpi_pm"); # improve timekeeping as well...
    my $grubby = "/sbin/grubby --update-kernel=/boot/vmlinuz-$kernel ";
    $grubby .= ($action eq "install" ? "--args=" : "--remove-args=") . "\"hda=noprobe hdb=noprobe $timekeeping\"";
    print "  Now run \"$grubby\"\n" if $debug;
    if (system($grubby)){
        #print "  Could not run $grubby: $!\n";
        $err++;
        next;
    }
    
    # Recreate initrd
    my $initrd = "/sbin/mkinitrd -f /boot/initrd-$kernel.img $kernel";
    $initrd .= " --with=vmbus --with=blkvsc --with=storvsc --with=netvsc" if $action eq "install";
    print "  Now run \"$initrd\"\n" if $debug;
    if (system($initrd)){
        #print "  Could not run $initrd: $!\n";
        $err++;
        next;
    }
    
    # Create a state file
    if ($action eq "install"){
        print "  Create file \"$statedir/$kernel\"\n" if $debug;
        system("/bin/touch $statedir/$kernel") && print "Could not create file \"$statedir/$kernel\"\n";
    }else{
        print "  Remove file \"$statedir/$kernel\"\n" if $debug;
        unlink "$statedir/$kernel";
    }
}

# Done

exit $err;

#
# Housekeeping: remove state file for kernels not found on the system
#
sub Clean(@){
    print "Housekeeping\n" if $debug;
    my @kernels = @_;
    if (opendir(STATE,$statedir)){
        for my $file (grep { ! /^\./ && -f "$statedir/$_" } readdir(STATE)){
	    if (not grep {$file eq $_} @all_kernels){
		unlink "$statedir/$file";
		print "  Remove file \"$statedir/$file\"\n" if $debug;
	    }
        }
        closedir STATE;
    }
    return;
}

#
# Print usage and exit
#
sub Usage(){
    print STDERR "Usage: $0 [ install | uninstall | clean ] [ kernel | kernel-xen | kernel-PAE ]\n";
    exit 1;
}
